@@ -1,8 +1,11 @@
+"""HTML escape test cases."""
+# pylint: disable=missing-class-docstring
 from unittest import TestCase
 from unittest import skipIf
 
 from typing import NamedTuple
 from typing import Dict
+from typing import List
 
 try:
     import markupsafe
@@ -11,10 +14,13 @@ except ImportError:
 
 try:
     from markupsafe import Markup
+    from markupsafe import escape
 except ImportError:
-    from liquid.exceptions import Markup
+    from liquid.exceptions import Markup  # type: ignore
+    from liquid.exceptions import escape  # type: ignore
 
 from liquid import Environment
+from liquid.exceptions import Error
 
 
 class Case(NamedTuple):
@@ -26,6 +32,18 @@ class Case(NamedTuple):
     expect: str
 
 
+class SafeHTMLDrop:
+    def __init__(self, somelist: List[object]):
+        self.items = somelist
+
+    def __str__(self):
+        return "SafeHTMLDrop"
+
+    def __html__(self):
+        lis = "\n".join(f"<li>{item}</li>" for item in self.items)
+        return f"<ul>\n{lis}\n</ul>"
+
+
 @skipIf(markupsafe is None, "this test requires markupsafe")
 class AutoescapeTestCase(TestCase):
     def test_do_not_escape_output(self):
@@ -36,7 +54,13 @@ class AutoescapeTestCase(TestCase):
                 template=r"{{ name }}",
                 context={"name": '<script>alert("XSS!");</script>'},
                 expect='<script>alert("XSS!");</script>',
-            )
+            ),
+            Case(
+                description="__html__ is ignored",
+                template=r"{{ foo }}",
+                context={"foo": SafeHTMLDrop([1, 2, 3])},
+                expect="SafeHTMLDrop",
+            ),
         ]
 
         env = Environment(autoescape=False)
@@ -85,6 +109,12 @@ class AutoescapeTestCase(TestCase):
                     "</p>"
                 ),
             ),
+            Case(
+                description="drops with __html__ are safe",
+                template=(r"{{ foo }}"),
+                context={"foo": SafeHTMLDrop([1, 2, 3])},
+                expect="<ul>\n<li>1</li>\n<li>2</li>\n<li>3</li>\n</ul>",
+            ),
         ]
 
         env = Environment(autoescape=True)
@@ -464,6 +494,12 @@ class AutoescapeTestCase(TestCase):
                 context={"some": Markup("&lt;p&gt;test&lt;/p&gt;<p>test</p>")},
                 expect="&lt;p&gt;test&lt;/p&gt;&lt;p&gt;test&lt;/p&gt;",
             ),
+            Case(
+                description="escape __html__",
+                template=r"{{ some | escape }}",
+                context={"some": SafeHTMLDrop([1, 2, 3])},
+                expect="SafeHTMLDrop",
+            ),
         ]
 
         env = Environment(autoescape=True)
@@ -588,6 +624,13 @@ class AutoescapeTestCase(TestCase):
                 result = template.render(**case.context)
                 self.assertEqual(result, case.expect)
 
+    def test_safe_filter_with_no_autoescape(self):
+        """Test that the safe filter is a no-op if autoescape is disabled."""
+        env = Environment(autoescape=False)
+        template = env.from_string(r"{{ '<p>Hello</p>' | safe }}")
+        result = template.render()
+        self.assertEqual(result, "<p>Hello</p>")
+
     def test_tablerow_tag(self):
         """Test that the tablerow tag behaves well with autoescape."""
         tests = [
@@ -600,14 +643,14 @@ class AutoescapeTestCase(TestCase):
                 ),
                 context={},
                 expect=(
-                    '<tr class="row1">'
+                    '<tr class="row1">\n'
                     '<td class="col1">1 true</td>'
                     '<td class="col2">2 false</td>'
-                    "</tr>"
+                    "</tr>\n"
                     '<tr class="row2">'
                     '<td class="col1">3 true</td>'
                     '<td class="col2">4 false</td>'
-                    "</tr>"
+                    "</tr>\n"
                 ),
             ),
             Case(
@@ -619,14 +662,14 @@ class AutoescapeTestCase(TestCase):
                 ),
                 context={},
                 expect=(
-                    '<tr class="row1">'
+                    '<tr class="row1">\n'
                     '<td class="col1"><b>1</b> true</td>'
                     '<td class="col2"><b>2</b> false</td>'
-                    "</tr>"
+                    "</tr>\n"
                     '<tr class="row2">'
                     '<td class="col1"><b>3</b> true</td>'
                     '<td class="col2"><b>4</b> false</td>'
-                    "</tr>"
+                    "</tr>\n"
                 ),
             ),
             Case(
@@ -637,12 +680,12 @@ class AutoescapeTestCase(TestCase):
                     r"{% endtablerow %}"
                 ),
                 expect=(
-                    '<tr class="row1">'
+                    '<tr class="row1">\n'
                     '<td class="col1">&lt;b&gt;tag1&lt;/b&gt;</td>'
                     '<td class="col2">tag2</td>'
                     '<td class="col3">tag3</td>'
                     '<td class="col4">tag4</td>'
-                    "</tr>"
+                    "</tr>\n"
                 ),
                 context={
                     "collection": {
@@ -663,12 +706,12 @@ class AutoescapeTestCase(TestCase):
                     r"{% endtablerow %}"
                 ),
                 expect=(
-                    '<tr class="row1">'
+                    '<tr class="row1">\n'
                     '<td class="col1"><b>tag1</b></td>'
                     '<td class="col2">tag2</td>'
                     '<td class="col3">tag3</td>'
                     '<td class="col4">tag4</td>'
-                    "</tr>"
+                    "</tr>\n"
                 ),
                 context={
                     "collection": {
@@ -739,3 +782,16 @@ class AutoescapeTestCase(TestCase):
                 template = env.from_string(case.template)
                 result = template.render(**case.context)
                 self.assertEqual(result, case.expect)
+
+
+@skipIf(markupsafe is not None, "this tests exceptions raise by the lack of markupsafe")
+class DummyMarkupSafeTestCase(TestCase):
+    def test_dummy_escape(self):
+        """Test that the dummy definition of escape raises an exception."""
+        with self.assertRaises(Error):
+            escape("foo")
+
+    def test_dummy_markup(self):
+        """Test that the dummy definition of Markup raises an exception."""
+        with self.assertRaises(Error):
+            Markup("foo")
